@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2013, Benjamin Drung <bdrung@debian.org>
+ * Copyright (C) 2012-2014, Benjamin Drung <bdrung@debian.org>
  *
  * Permission to use, copy, modify, and/or distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -103,7 +103,7 @@ static inline bool is_valid_date(const date_t *date) {
     return date->month >= 1 && date->month <= 12 &&
            date->day >= 1 &&
            date->day <= (is_leap_year(date->year) &&
-           date->month == 1 ? 29 : days_in_month[date->month-1]);
+           date->month == 2 ? 29 : days_in_month[date->month-1]);
 }
 
 static time_t date_to_secs(const date_t *date) {
@@ -261,6 +261,20 @@ static const distro_t *select_latest_release(const distro_elem_t *distro_list) {
     return selected;
 }
 
+static const distro_t *select_series(const distro_elem_t *distro_list, const char *series) {
+    const distro_t *selected;
+
+    selected = NULL;
+    while(distro_list != NULL) {
+        if(distro_list && (strcmp(distro_list->distro->series, series) == 0)) {
+            selected = distro_list->distro;
+            break;
+        }
+        distro_list = distro_list->next;
+    }
+    return selected;
+}
+
 static bool calculate_days(const distro_t *distro, const date_t *date,
                            int date_index, ssize_t *days) {
     const date_t *first;
@@ -515,6 +529,7 @@ static void print_help(void) {
            "Options:\n"
            "  -h  --help             show this help message and exit\n"
            "      --date=DATE        date for calculating the version (default: today)\n"
+           "      --series=SERIES    series to calculate the version for\n"
            "  -y[MILESTONE]          additionally, display days until milestone\n"
            "      --days=[MILESTONE] ("
           );
@@ -559,7 +574,7 @@ static inline int not_exactly_one(void) {
 #endif
             "--all, --devel, "
 #ifdef UBUNTU
-            "--lts, "
+            "--latest, --lts, "
 #endif
 #ifdef DEBIAN
             "--oldstable, "
@@ -568,6 +583,7 @@ static inline int not_exactly_one(void) {
 #ifdef UBUNTU
             "--supported-esm, "
 #endif
+            "--series, "
 #ifdef DEBIAN
             "--testing, "
 #endif
@@ -584,6 +600,7 @@ int main(int argc, char *argv[]) {
     const distro_t *selected;
     int i;
     int date_index = -1;
+    char *series_name = NULL;
     int option;
     int option_index;
     int return_value = EXIT_SUCCESS;
@@ -594,35 +611,40 @@ int main(int argc, char *argv[]) {
 #ifdef DEBIAN
     char *alias_codename = NULL;
 #endif
+#ifdef UBUNTU
+    bool filter_latest = false;
+#endif
 
     const struct option long_options[] = {
-        {"help",        no_argument,       NULL, 'h' },
-        {"date",        required_argument, NULL, 'D' },
-        {"all",         no_argument,       NULL, 'a' },
-        {"days",        optional_argument, NULL, 'y' },
-        {"devel",       no_argument,       NULL, 'd' },
-        {"stable",      no_argument,       NULL, 's' },
-        {"supported",   no_argument,       NULL, 'S' },
+        {"help",          no_argument,       NULL, 'h' },
+        {"date",          required_argument, NULL, 'D' },
+        {"series",        required_argument, NULL, 'R' },
+        {"all",           no_argument,       NULL, 'a' },
+        {"days",          optional_argument, NULL, 'y' },
+        {"devel",         no_argument,       NULL, 'd' },
+        {"stable",        no_argument,       NULL, 's' },
+        {"supported",     no_argument,       NULL, 'S' },
 #ifdef UBUNTU
-        {"supported-esm", no_argument,     NULL, 'e' },
+        {"supported-esm", no_argument,       NULL, 'e' },
 #endif
-        {"unsupported", no_argument,       NULL, 'U' },
-        {"codename",    no_argument,       NULL, 'c' },
-        {"fullname",    no_argument,       NULL, 'f' },
-        {"release",     no_argument,       NULL, 'r' },
+        {"unsupported",   no_argument,       NULL, 'U' },
+        {"codename",      no_argument,       NULL, 'c' },
+        {"fullname",      no_argument,       NULL, 'f' },
+        {"release",       no_argument,       NULL, 'r' },
 #ifdef DEBIAN
-        {"alias",       required_argument, NULL, 'A' },
-        {"oldstable",   no_argument,       NULL, 'o' },
-        {"testing",     no_argument,       NULL, 't' },
+        {"alias",         required_argument, NULL, 'A' },
+        {"oldstable",     no_argument,       NULL, 'o' },
+        {"testing",       no_argument,       NULL, 't' },
 #endif
 #ifdef UBUNTU
-        {"lts",         no_argument,       NULL, 'L' },
+        {"latest",        no_argument,       NULL, 'l' },
+        {"lts",           no_argument,       NULL, 'L' },
 #endif
-        {NULL,          0,                 NULL, '\0' }
+        {NULL,            0,                 NULL, '\0' }
     };
 
 #ifdef UBUNTU
-    const char *short_options = "hadscrfy::";
+    const char *short_options = "hadscrfly::";
 #endif
 #ifdef DEBIAN
     const char *short_options = "hadscrfoty::";
@@ -692,6 +714,22 @@ int main(int argc, char *argv[]) {
                 }
                 break;
 
+            case 'R':
+                // Only long option --series is used
+                if(unlikely(series_name != NULL)) {
+                    fprintf(stderr, NAME ": series requested multiple times.\n");
+                    return EXIT_FAILURE;
+                }
+                if(!optarg || !is_valid_codename(optarg)) {
+                    fprintf(stderr, NAME ": invalid distribution series `%s'\n",
+                            optarg);
+                    free(date);
+                    return EXIT_FAILURE;
+                }
+                selected_filters++;
+                series_name = optarg;
+                break;
+
             case 'f':
                 just_days = false;
                 print_cb = print_fullname;
@@ -703,6 +741,13 @@ int main(int argc, char *argv[]) {
                 return EXIT_SUCCESS;
 
 #ifdef UBUNTU
+            case 'l':
+                selected_filters++;
+                filter_latest = true;
+                filter_cb = filter_devel;
+                select_cb = select_latest_created;
+                break;
+
             case 'L':
                 // Only long option --lts is used
                 selected_filters++;
@@ -786,6 +831,9 @@ int main(int argc, char *argv[]) {
                 } else if(optopt == 'D') {
                     fprintf(stderr, NAME ": option `--date' requires "
                             "an argument DATE\n");
+                } else if(optopt == 'R') {
+                    fprintf(stderr, NAME ": option `--series' requires "
+                            "an argument SERIES\n");
                 } else {
                     fprintf(stderr, NAME ": unrecognized option `-%c'\n",
                             optopt);
@@ -862,12 +910,25 @@ int main(int argc, char *argv[]) {
     }
 #endif
 
-    if(select_cb == NULL) {
+    if(select_cb == NULL && !series_name) {
         filter_data(distro_list, date, date_index, just_days, filter_cb, print_cb);
     } else {
-        selected = get_distro(distro_list, date, filter_cb, select_cb);
+        if(series_name) {
+            selected = select_series(distro_list, series_name);
+        } else {
+            selected = get_distro(distro_list, date, filter_cb, select_cb);
+#ifdef UBUNTU
+            if(selected == NULL && filter_latest) {
+                selected = get_distro(distro_list, date, filter_stable, select_latest_release);
+            }
+#endif
+        }
         if(selected == NULL) {
-            fprintf(stderr, NAME ": " OUTDATED_ERROR "\n");
+            if(series_name) {
+                fprintf(stderr, NAME ": unknown distribution series `%s'\n", series_name);
+            } else {
+                fprintf(stderr, NAME ": " OUTDATED_ERROR "\n");
+            }
             return_value = EXIT_FAILURE;
         } else {
             if(!print_cb(selected, date, date_index, just_days)) {
